@@ -1,5 +1,6 @@
 from collections import defaultdict
 from pyclbr import Function
+from random import random
 from tokenize import Double
 from typing import Dict
 from xmlrpc.client import Boolean
@@ -47,7 +48,7 @@ class MVNMixtureModel():
         self.params = {"N":self._N, "K":self.K, "T":self._T}
         self.init_params = {"N":self._N, "K":self.K, "T":self._T, \
             "sigma":None, "mean":None, "weights":None, \
-            "clusters":None, "var_constr":None}
+            "clusters":None, "var_constr":None, "is_computed":None}
         self.hyperparameters = { \
             "mean_scale":min(self.dataset.float().var(), torch.tensor(1000).float()), \
             "mean_loc":self.dataset.float().max() / 2, \
@@ -302,7 +303,7 @@ class MVNMixtureModel():
                     sigma_chol[k]).add(torch.eye(self._T))
         return Sigma
 
-    
+
     def _initialize_centroids(self, K, N, random_state):
         km = KMeans(n_clusters=K, random_state=random_state).fit(self.dataset)
         self.init_params["clusters"] = torch.from_numpy(km.labels_)
@@ -368,25 +369,28 @@ class MVNMixtureModel():
         as the proportion of IS assigned to each cluster and the
         Poisson lambda or Gaussian mean as the centers of each cluster.
         '''
-        N, K = self.params["N"], self.params["K"]
+        if not self.init_params["is_computed"]:
+            N, K = self.params["N"], self.params["K"]
 
-        ctrs = self._initialize_centroids(K, N, random_state)
-        var, var_constr = self._initialize_variance(K, ctrs)
-        sigma_chol = self._initialize_sigma_chol(K)
-        Sigma = self.compute_Sigma(sigma_chol=sigma_chol, sigma_vector=var, K=K)
+            ctrs = self._initialize_centroids(K, N, random_state)
+            var, var_constr = self._initialize_variance(K, ctrs)
+            sigma_chol = self._initialize_sigma_chol(K)
+            Sigma = self.compute_Sigma(sigma_chol=sigma_chol, sigma_vector=var, K=K)
 
-        self.init_params["mean"] = ctrs
-        self.init_params["sigma_vector"] = var
-        self.init_params["var_constr"] = var_constr
-        self.init_params["sigma_chol"] = sigma_chol
-        self.init_params["sigma"] = Sigma
+            self.init_params["mean"] = ctrs
+            self.init_params["sigma_vector"] = var
+            self.init_params["var_constr"] = var_constr
+            self.init_params["sigma_chol"] = sigma_chol
+            self.init_params["sigma"] = Sigma
+
+            self.init_params["is_computed"] = True
         
         return self.init_params
 
 
     def fit(self, steps=500, optim_fn=pyro.optim.Adam, lr=0.001, cov_type="diag", \
             loss_fn=pyro.infer.TraceEnum_ELBO(), convergence=True, initializ=True, \
-            min_steps=1, p=0.05, random_state=25, store_params=False, show_progr=True):
+            min_steps=1, p=0.01, random_state=25, store_params=False, show_progr=True):
         pyro.enable_validation(True)
         pyro.clear_param_store()
 
@@ -396,6 +400,9 @@ class MVNMixtureModel():
         self.cov_type = cov_type
         self._seed = random_state
 
+        if random_state is not None:
+            pyro.set_rng_seed(random_state)
+
         if initializ:
             # set the guide and initialize the SVI object to minimize the initial loss 
             loss, self._seed = min((self._initialize(seed), seed) for seed in range(100))
@@ -504,19 +511,20 @@ class MVNMixtureModel():
         return 0
 
 
-    def _check_convergence(self, par, perc=0.01) -> Boolean:
+    def _check_convergence(self, par, p=0.01) -> Boolean:
         '''
         - `par` -> list of 2 elements given the previous (at index 0) and current (at index 1) 
         estimated values for a parameter.
-        - `perc` -> numeric value in `[0,1]`.
+        - `p` -> numeric value in `[0,1]`, corresponding to a percentage of the learning rate used for convergence.
         The function returns `True` if more than 95% of the values changed less than `perc*100`% 
         in the current step with respect to the previous one.
         '''
+        eps = p * self._settings["lr"]
         n = 0
         for k in range(par[0].shape[0]):
             for t in range(par[0].shape[1]):
-                p = perc * torch.max(torch.tensor(1), par[0][k,t])
-                if torch.absolute(par[0][k,t] - par[1][k,t]) <= p:
+                perc = eps * torch.max(torch.tensor(1), par[0][k,t])
+                if torch.absolute(par[0][k,t] - par[1][k,t]) <= perc:
                     n += 1
         return n >= .99 * self.params["K"]*self.params["T"]
 
